package com.snowm.scanner

import android.content.Context
import android.content.SharedPreferences
import android.os.Build
import android.os.Bundle
import androidx.annotation.NonNull
import androidx.annotation.RequiresApi
import com.google.gson.Gson
import com.snowm.scanner.helpers.*
import com.snowm.scanner.helpers.SnowMBackgroundScanner.Companion.startBackgroundScanning
import com.snowm.scanner.helpers.SnowMBackgroundScanner.Companion.stopBackgroundScanning
import com.snowm.scanner.models.Geofence
import com.snowm.scanner.models.SnowMiBeacon
import com.snowm.scanner.services.SnowMBackgroundScanningService
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import org.altbeacon.beacon.Region
import kotlin.collections.ArrayList
import kotlin.collections.HashMap
import kotlin.properties.Delegates

class SnowmScannerPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    private lateinit var channel: MethodChannel
    private lateinit var beaconScanner: BeaconScanner
    lateinit var mqttHelper: MqttHelper
    private lateinit var permissionManager: PermissionManager
    private lateinit var context: Context
    private lateinit var geofencingHelper: GeofencingHelper
    private lateinit var sharedPreferences: SharedPreferences


    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "snowm_scanner")
        channel.setMethodCallHandler(this)
    }

    private fun initilizeWithContext() {
        mqttHelper = MqttHelper(context)
        beaconScanner = BeaconScanner(context)
        permissionManager = PermissionManager(context)
        geofencingHelper = GeofencingHelper(context)
        sharedPreferences = context.getSharedPreferences("snowm_scanner", Context.MODE_PRIVATE)
    }

    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "snowm_scanner")
            channel.setMethodCallHandler(SnowmScannerPlugin())
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        val method: String = call.method
        when {
            method.contains("scanBeacons") -> {
                val uuids: ArrayList<String> = call.argument("uuids")!!
                val enableMqtt: Boolean = call.argument<Boolean>("enableMqtt")!!
                val timeInterval: Int = call.argument<Int>("timeInterval")!!
                mqttHelper.timeInterval = timeInterval
                mqttHelper.enabled = enableMqtt
                if (permissionManager.hasLocationPermission()) {
                    scan(uuids, method)
                } else {
                    permissionManager.requestLocationPermission()
                    scan(uuids, method)
                }
                result.success(true)
            }
            method == "scanBeaconBackground" -> {
                val uuids: ArrayList<String> = call.argument("uuids")!!
                val customData: HashMap<String, Any> = call.argument("customData")!!
                val title: String = call.argument("title")!!
                val body: String = call.argument("body")!!
                val scanId: String = call.argument("scanId")!!
                val backgroundBetweenScanPeriod:Long = call.argument("backgroundBetweenScanPeriod")!!
                var backgroundScanPeriod :Long = call.argument("backgroundScanPeriod")!!
                val geofences: ArrayList<HashMap<String, Any>> = call.argument("geofences")!!

                val bundle = Bundle()
                bundle.putString("title", title)
                bundle.putString("body", body)
                bundle.putString("scanId", scanId)
                bundle.putLong("backgroundBetweenScanPeriod",backgroundBetweenScanPeriod)
                bundle.putLong("backgroundScanPeriod",backgroundScanPeriod)
                bundle.putSerializable("customData", customData)
                bundle.putSerializable("geofences", geofences)
                bundle.putStringArrayList("uuids", uuids)

                sharedPreferences.edit().putString("backgroundScanner", Gson().toJson(bundle)).apply()

                if (!permissionManager.hasLocationPermission())
                    permissionManager.requestLocationPermission()
                startBackgroundScanning(context, bundle)
                geofences.forEach {
                    val geofence = Geofence(it)
                    geofencingHelper.stopGepfencing(geofence.identifier)
                    geofencingHelper.startGeoFencing(geofence)
                }
                result.success(true)
            }
            method == "stopBackgroundScan" -> {
                sharedPreferences.edit().remove("backgroundScanner").apply()
                stopBackgroundScanning(context)
                geofencingHelper.stopGepfencing("backgroundGeofence")
                result.success(true)
            }
            method == "requestPermission" -> {
                result.success(permissionManager.requestLocationPermission())
            }
            method == "permissionState" -> {
                result.success(permissionManager.bluetoothPermissionState)
            }
            method == "bluetoothState" -> {
                result.success(permissionManager.bluetoothState)
            }
            method.contains("bluetoothStateListener") -> {
                permissionManager.bluetoothState(method, object : PermissionManager.BluetoothStateListener {

                    override fun onStateChanged(bluetoothState: PermissionManager.BluetoothState) {
                        sendBluetoothResponse(method, bluetoothState.state)
                    }
                })
                result.success(true)
            }
            method == "cancelStream" -> {
                val streamIdentifier = call.argument<String>("methodName")!!
                beaconScanner.stopRanging(streamIdentifier)
                result.success(true)
            }
            method == "cancelBluetoothStateListener" -> {
                val streamIdentifier = call.argument<String>("methodName")!!
                permissionManager.removeBluetoothState(streamIdentifier)
                result.success(true)
            }
            method == "getCurrentScanId" -> {
                val scanId = SnowMBackgroundScanningService.scanId;
                result.success(scanId)
            }
            else -> {
                print("method not implemented")
                result.notImplemented()
            }
        }
    }

    private fun scan(uuids: ArrayList<String>, method: String) {
        beaconScanner.rangeBeacons(method,0L,0L)
        beaconScanner.setOnBeaconScanner(method,
                object : BeaconScanner.OnBeaconScanned {
                    override fun onBeaconRanged(beacons: ArrayList<SnowMiBeacon>) {
                        val validBeacons: ArrayList<SnowMiBeacon> = ArrayList()
                        beacons.forEach() {
//                            mqttHelper.sendIBeaconInformation(it)
                            if (uuids.contains(it.uuid.toUpperCase()))
                                validBeacons.add(it)
                        }
                        sendBeaconResponse(method, validBeacons)
                    }

                    override fun didEnterRegion(p0: Region?) {
                    }

                    override fun didExitRegion(p0: Region?) {
                    }
                })
    }


    private fun sendBeaconResponse(method: String, beacons: ArrayList<SnowMiBeacon>) {
        val response: HashMap<String, Any> = HashMap()
        response["beacons"] = beacons.map { b -> b.toObject() }
        channel.invokeMethod(method, response)
    }

    private fun sendBluetoothResponse(method: String, state: String) {
        val response: String = state
        channel.invokeMethod(method, response)
    }


    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onDetachedFromActivity() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.context = binding.activity
        initilizeWithContext()
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }


}

