import Foundation
import CoreLocation

public class BackgroundBeaconScanner: NSObject, CLLocationManagerDelegate{
    
    public static let shared = BackgroundBeaconScanner();
    open var delegate: SnowMBackgroundScanningServiceDelegate?
    
    private let locationManager = CLLocationManager()
    
    
    private var customData: Dictionary<String,Any>?
    private var uuids: [String]?
    
    override init(){
        super.init()
        locationManager.pausesLocationUpdatesAutomatically = false
        locationManager.delegate = self;
    }
    
    public func scanBeacons(scanId:String, uuids:[String], customData: Dictionary<String,Any>){
        locationManager.requestAlwaysAuthorization()
        locationManager.startUpdatingLocation()
        UserDefaults.standard.set(scanId, forKey: "currentScannerId")
        UserDefaults.standard.setValue(uuids, forKey: "uuids")
        do{
            let data = try JSONSerialization.data(withJSONObject: customData, options: .prettyPrinted)
            let jsonData = String(data: data, encoding: .ascii)
            UserDefaults.standard.setValue(jsonData, forKey: "customData")
        } catch {
            print(error)
        }
        startBeaconRanging()
        delegate?.onScannerRegistered(scanId: scanId, customData: customData)
    }
    
    public func currentScannerId()-> String? {
        return UserDefaults.standard.string(forKey: "currentScannerId")
    }
    
    public func startBeaconRanging(){
        getDataFromStorage()
        uuids!.forEach { (id) in
            let uuid = UUID(uuidString: id)
            let region = CLBeaconRegion(proximityUUID: uuid!, identifier: id)
            locationManager.startMonitoring(for: region)
            locationManager.startRangingBeacons(in: region)
        }
        
    }
    
    public func addGeoFencing(identifier:String, location: CLLocationCoordinate2D, radius: CLLocationDistance){
        let region = CLCircularRegion(center: location, radius: radius, identifier: identifier)
        region.notifyOnExit = true
        region.notifyOnEntry = true
        locationManager.startMonitoring(for: region)
    }
    
    public func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        delegate?.onLocationUpdated(locations: locations)
    }
    
    public func removeBeaconRanging(){
        for region in locationManager.rangedRegions {
            if(region is CLBeaconRegion){
                locationManager.stopRangingBeacons(in: region as! CLBeaconRegion)
            }
        }
    }
    
    public func removeLocationUpdate(){
        locationManager.stopUpdatingLocation()
    }
    
    public func removeBeaconMonitoring(){
        for region in locationManager.monitoredRegions {
            if(region is CLBeaconRegion){
                locationManager.stopMonitoring(for: region)
            }
        }
        UserDefaults.standard.set(nil, forKey: "currentScannerId")
    }
    
    public func removeGeofencing(identifier:String){
        for region in locationManager.monitoredRegions {
            if(region is CLCircularRegion){
                if(region.identifier == identifier){
                    locationManager.stopMonitoring(for: region)
                }
            }
        }
    }
    
    public func stopBackgroundScanning(){
        removeBeaconMonitoring()
        removeLocationUpdate()
        removeBeaconRanging()
        removeGeofencing(identifier: "backgroundGeofence")
    }
    
    public func locationManager(_ manager: CLLocationManager, didEnterRegion region: CLRegion) {
        if region is CLCircularRegion{
            delegate?.didEnterGeofence(region: region as! CLCircularRegion);
        } else if region is CLBeaconRegion{
            delegate?.onRegionEntered(region: region as! CLBeaconRegion)
        }
    }
    
    public func locationManager(_ manager: CLLocationManager, didExitRegion region: CLRegion) {
        if region is CLCircularRegion{
            delegate?.didExitGeofence(region: region as! CLCircularRegion)
        } else {
            delegate?.onRegionExited(region: region as! CLBeaconRegion)
        }
    }
    
    private func getDataFromStorage(){
        uuids = UserDefaults.standard.stringArray(forKey: "uuids")!
        let jsonData: String? = UserDefaults.standard.string(forKey: "customData")
        if(jsonData != nil){
            do{
                let data = jsonData?.data(using: .utf8)
                let decoded = try JSONSerialization.jsonObject(with: data!, options: [])
                customData = decoded as? Dictionary<String, Any>
            } catch{
                print(error)
            }
        }
    }
    
    func getUuidToBeScanned() ->  [String]?{
        getDataFromStorage()
        return uuids
    }
    
    func getCustomData() ->Dictionary<String,Any>?{
        getDataFromStorage()
        return customData
    }
    
    
    public func locationManager(_ manager: CLLocationManager, didRangeBeacons beacons: [CLBeacon], in region: CLBeaconRegion) {
        if (beacons.count==0) {return}
        let snowmBeacons:[SnowMBeacon] = beacons.map {
            let snowmBeacon = SnowMBeacon()
            snowmBeacon.uuid = $0.proximityUUID.uuidString
            snowmBeacon.major = $0.major.intValue
            snowmBeacon.minor = $0.minor.intValue
            snowmBeacon.txPower = 0
            snowmBeacon.macAddress = "Unavailable"
            snowmBeacon.distance = getDistance(rssi: $0.rssi)
            snowmBeacon.rssi = $0.rssi
            return snowmBeacon
        }
        delegate?.onBeaconDetected(beacons: snowmBeacons, customData: customData)
    }
    
    func getDistance(rssi:Int) -> Double{
        let mesauredPower = -65;
        let environmentalFactor = getEnvironmentalFactor(rssi:rssi)
        let core: Double = (Double(mesauredPower - rssi)) / (10.0 * environmentalFactor)
        let distance = Double(10) ^^ Double(core);
        return distance
    }
    
    func getEnvironmentalFactor(rssi:Int) -> Double{
        if(rssi < -100){ return 2}
        if(rssi > -35) {return 4}
        let range: Double = Double(rssi+100) / Double(65)
        return 2.0 + 2.0 * range
    }
    
    public func locationManager(_ manager: CLLocationManager, rangingBeaconsDidFailFor region: CLBeaconRegion, withError error: Error) {
    }
}

precedencegroup PowerPrecedence { higherThan: MultiplicationPrecedence }
infix operator ^^ : PowerPrecedence
func ^^ (radix: Double, power: Double) -> Double {
    return Double(pow(radix, power))
}
